#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from taz.acr import ContainerRegistry
from taz.acr import ContainerImage
import tests.config as cfg
import sys


class AcrTests(unittest.TestCase):

    def setUp(self):
        # self.managed_identity = MsiAuthentication()

        self.container_registry = ContainerRegistry(
            cfg.acr["resource_group"],
            cfg.acr["registry_name"],
            subscription_id=cfg.acr["subscription_id"])

        self.container_image = ContainerImage(
            cfg.acr["image_name"],
            self.container_registry)

    def test_exists(self):
        self.assertTrue(self.container_registry is not None)
        self.assertTrue(self.container_image is not None)

    def test_get_credentials(self):
        self.assertTrue(self.container_registry.get_credentials() is not None)


if __name__ == '__main__':
    sys.argv.append('-v')
    unittest.main()



#!/usr/bin/env python3
# coding: utf8
# -----------------------------------------------------------------
# hermes: __init__.py
#
# Copyright (C) 2018-2019, Christophe Fauchard
# -----------------------------------------------------------------
"""
Module: taz

Trivial Azure API

Copyright (C) 2018-2019, Christophe Fauchard
"""

import sys
from taz._version import __version__, __version_info__

__author__ = "Christophe Fauchard <christophe.fauchard@gmail.com>"

if sys.version_info < (3, 6):
    raise RuntimeError('You need Python 3.6+ for this module.')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Taz library: Container Registry operations
"""

from azure.common.client_factory import get_client_from_cli_profile
from azure.mgmt.containerregistry import ContainerRegistryManagementClient
from msrestazure.azure_active_directory import MSIAuthentication
from azure.mgmt.containerinstance.models import ImageRegistryCredential


class ContainerRegistry:

    """Summary
    
    Attributes:
        - client (TYPE): Description
        - credentials (TYPE): Description
        - name (TYPE): Description
        - resource_group (TYPE): Description
    """
    
    def __init__(self,
                 resource_group,
                 name,
                 subscription_id=None):

        try:
            if subscription_id is None:
                self.client = get_client_from_cli_profile(
                    ContainerRegistryManagementClient)
            else:
                self.client = get_client_from_cli_profile(
                    ContainerRegistryManagementClient,
                    subscription_id=subscription_id)

        except:
            self.client = ContainerRegistryManagementClient(
                MSIAuthentication())

        self.name = name
        self.resource_group = resource_group
        self.credentials = self.client.registries.list_credentials(
            resource_group, name)

    def get_credentials(self):
        return ImageRegistryCredential(
            server="{0}.azurecr.io".format(self.name),
            username=self.credentials.username,
            password=self.credentials.passwords[0].value)


class ContainerImage:

    """Summary

    Attributes:
        container_registry (TYPE): Description
        name (TYPE): Description
    """

    def __init__(self,
                 name,
                 container_registry):

        self.container_registry = container_registry
        self.name = name

    def get_name(self):
        return(self.name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import pandas as pd
from azure.datalake.store import core, lib
from adal.adal_error import AdalError


class Datalake:
    def __init__(
        self,
        name,
        tenant_id,
        client_id,
        secret,
        proxy=None
    ):
        """
        Datalake connection

        Args:
            - name (string): Datalake name
            - tenant_id (string): Azure tenant id
            - client_id (string): service principal client id
            - secret (TYPE): service principal password
            - proxy (None, optional): HTTPS proxy
        """

        if proxy is not None:
            self.proxy = proxy
            os.environ['https_proxy'] = proxy

        try:
            self.token = lib.auth(
                tenant_id=tenant_id,
                client_secret=secret,
                client_id=client_id,
                resource='https://management.core.windows.net/')
        except AdalError as error:
            raise error

        self.conn = core.AzureDLFileSystem(
            self.token,
            store_name=name)

    def ls(self, path):
        """list DLS path
        
        Args:
            - path (string): path to list
        
        Returns:
            - (list) of dictionnaries: node entry metadatas with folowing keys:
                - 'length'
                - 'pathSuffix'
                - 'type'
                - 'blockSize'
                - 'accessTime'
                - 'modificationTime'
                - 'replication'
                - 'permission'
                - 'owner'
                - 'group'
                - 'msExpirationTime'
                - 'aclBit'
                - 'name'
        """
        return self.conn.ls(path, detail=True)

    def glob(self, path, details=True):
        """list DLS path with globbing
        
        Args:
            - path (string): path to list
        
        Returns:
            - (list) of dictionnaries: node entry metadatas with folowing keys:
                - 'length'
                - 'pathSuffix'
                - 'type'
                - 'blockSize'
                - 'accessTime'
                - 'modificationTime'
                - 'replication'
                - 'permission'
                - 'owner'
                - 'group'
                - 'msExpirationTime'
                - 'aclBit'
                - 'name'
        """
        return self.conn.glob(path, details=True)

    def get(self, srcpath, destpath):
        """download file from Datalake
        
        Args:
            srcpath (string): source path
            destpath (string): local destination path
        
        Returns:
            TYPE: Description
        """
        return self.conn.get(srcpath, destpath)

    def open(self, path):
        """open a stream descriptor on remote file
        
        Args:
            path (string): file path to open
        
        Returns:
            azure.datalake.store.core.AzureDLFile: Description
        """
        return self.conn.open(path)

    def read_csv(self, path, **kargs):
        """read CSV file from Datalake
        
        Args:
            - path (string): remote csv file path to read
            - **kargs: arguments array passed to pandas.read_csv
        
        Returns:
            - pandas.DataFrame: DataFrame filled with read datas
        """
        fd = self.open(path)
        df = pd.read_csv(fd, **kargs)
        fd.close()
        return df

    def df(self, path):
        """describe 
        
        Args:
            path (TYPE): Description
        
        Returns:
            TYPE: Description
        """
        stats = self.conn.df(path)
        return list(
            [
                stats['directoryCount'],
                stats['fileCount'],
                stats['spaceConsumed'] / 1024 / 1024 / 1024
            ])



keyvault = {
    "name": "kvdvldev",
    "secret_name": "REGISTRY-PASSWORD",
    "tenant_id": "4a7c8238-5799-4b16-9fc6-9ad8fce5a7d9",
    "client_id": "9d6f83f7-af2f-4a8d-b692-e2329dbbef9d",
    "client_secret": "C89nigG4BeonI",
}

auth = {
    "resource_group": "RG-APPLI-DVL-DEV",
    "managed_identity": "mi-dvl-dev-001",
    "subscription_id": "146f1dc4-e9bd-4cd1-90b8-4d9701f550f2",
    "tenant_id": "4a7c8238-5799-4b16-9fc6-9ad8fce5a7d9",
    "client_id": "9d6f83f7-af2f-4a8d-b692-e2329dbbef9d",
    "client_secret": "C89nigG4BeonI",
}

blob = {
    "storage_account_name": "zstalrsdvllogdev01",
    "storage_key": "z++1cFPTnrNBNIM6QONcnjYUXa1VZuCs6DpsP+x9ELEuItoAZyM/W+FQLqlxsy/DWfjxywsJu5FjW2O7Wc3P1A==",
    "sas_key": "?sv=2019-02-02&ss=bfqt&srt=sco&sp=rwdlacup&se=2019-12-05T02:24:33Z&st=2019-12-04T18:24:33Z&spr=https&sig=MssPnv0uBl%2BoWxplggnsUBlZLRi8lBWgMd1dy%2BnVo3Q%3D",
    "container_name": "unittest-taz",
    "path": "test.csv",
    "gzip_path": "test.csv.gz",
    "data": "col1,col2,col3\nval11,val12,val13\nval21,val22,val23",
}

acr = {
    "resource_group": "RG-APPLI-DVL-DEV",
    "registry_name": "crdvldev",
    "subscription_id": "146f1dc4-e9bd-4cd1-90b8-4d9701f550f2",
    "image_name": "hello-world",
}

aci = {
    "resource_group": "RG-APPLI-DVL-DEV",
    "container_group_name": "cg-taz-test-unit",
    "location": "northeurope",
    "subscription_id": "146f1dc4-e9bd-4cd1-90b8-4d9701f550f2",
}

dls = {
    "dls_name": "lemanhprod",
    "tenant_id": "4a7c8238-5799-4b16-9fc6-9ad8fce5a7d9",
    "client_id": "e3952129-496b-416b-a244-67c43bd5bddd",
    "client_secret": "/KauQDlt2yiL34n9xsCxPLJMgKVzf.==",
    "https_proxy": "localhost:3128",
    "path": "SNCF/landing/input/METEOFRANCE/data_files/OBSERVATIONS_5MN/2019",
    "glob_path": "SNCF/landing/input/METEOFRANCE/data_files/OBSERVATIONS_5MN/2019/matriceSNCF_20191212*",
    "file_name": "matriceSNCF_201905201520.csv",
    "gz_file_name": "SNCF/landing/input/METEOFRANCE/data_files/PREVISIONS/20190123/previsions_20190123_J1.csv.gz",
    "tmp_file_name": "/tmp/taz.tmp",
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from taz.auth import UserAssignedIdentity, ClientSecretAuthentication
import sys
import os
import tests.config as cfg


class AuthTests(unittest.TestCase):
    def setUp(self):
        # self.managed_identity = MsiAuthentication()

        self.user_assigned_identity = UserAssignedIdentity(
            cfg.auth["resource_group"],
            cfg.auth["managed_identity"],
            subscription_id=cfg.auth["subscription_id"],
        )

    def test_exists(self):
        self.assertTrue(self.user_assigned_identity is not None)

    def test_secret_credentials_by_params(self):
        self.client_secret_credentials = ClientSecretAuthentication(
            cfg.auth["tenant_id"],
            cfg.auth["subscription_id"],
            cfg.auth["client_id"],
            cfg.auth["client_secret"],
        )
        print(self.client_secret_credentials.get_credentials())

    def test_secret_credentials_by_env(self):
        os.environ["AZURE_TENANT_ID"] = cfg.auth["tenant_id"]
        os.environ["AZURE_SUBSCRIPTION_ID"] = cfg.auth["subscription_id"]
        os.environ["AZURE_CLIENT_ID"] = cfg.auth["client_id"]
        os.environ["AZURE_CLIENT_SECRET"] = cfg.auth["client_secret"]

        self.client_secret_credentials = ClientSecretAuthentication()
        print(self.client_secret_credentials.get_credentials())

    def test_container_group_identity(self):
        self.assertTrue(
            self.user_assigned_identity.get_container_group_identity() is not None
        )

    def test_managed_identity(self):
        self.assertTrue(self.user_assigned_identity is not None)


if __name__ == "__main__":
    sys.argv.append("-v")
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Taz library: blobs operations
"""


from azure.storage.common.cloudstorageaccount import CloudStorageAccount
import urllib
from azure.storage.blob import BlobPermissions
from datetime import datetime, timedelta
import pandas as pd
import gzip


class StorageAccount:
    def __init__(
        self, name, storage_key=None,
    ):
        """ Storage account object

        Args:
            name (TYPE): Description
            storage_key (None, optional): Description
        """
        self.storage_client = CloudStorageAccount(name, storage_key)

        self.name = name
        self.blob_service = self.storage_client.create_block_blob_service()
        self.page_blob_service = self.storage_client.create_page_blob_service()

    def list_containers(self):
        """ list storage account containers

        Returns:
            - azure.storage.blob.models.Container List: list of container, Container class attributes:
                - name
                - metadata
                - properties
        """
        return self.blob_service.list_containers()

    def list_blobs(self, container, prefix=None):
        """ list blobs in specified container

        Args:
            - container (string): container
            - prefix (string, optional): prefix of blobs to select

        Returns:
            - azure.storage.blob.models.Blob: Blob class attributes
                - name: blob name
                - snapshot
                - content
                - properties (BlobProperties)
                - deleted
        """
        return self.blob_service.list_blobs(container, prefix=prefix)

    def copy_blob(self, container, blob, file):
        """copy blob content to local file

        Args:
            container (string): container name
            blob (string): blob name
            file (string): local file name

        Returns:
            azure.storage.blob.models.Blob: blob copied
        """
        return self.page_blob_service.get_blob_to_path(container, blob, file,)


class Container:
    def __init__(self, storage_account, name):
        """ define Container object 

        Args:
            storage_account (TYPE): Description
            name (TYPE): Description
        """
        self.storage_account = storage_account
        self.name = name

    def create(self):
        return self.storage_account.blob_service.create_container(self.name)

    def delete(self):
        return self.storage_account.blob_service.delete_container(self.name)


class Blob:
    def __init__(
        self, storage_account, container, name, sas_key=None,
    ):
        self.storage_account = storage_account
        self.container = container
        self.name = name
        self.sas_key = sas_key

    def get_sas_token(self):
        permission = BlobPermissions(_str="racwd")
        self.sas_token = self.storage_account.blob_service.generate_blob_shared_access_signature(
            self.container,
            self.name,
            permission,
            datetime.utcnow() + timedelta(hours=1),
        )
        return self.sas_token

    def get_url(self):
        return self.storage_account.blob_service.make_blob_url(
            self.container, self.name, protocol="https", sas_token=self.get_sas_token()
        )

    def read(self):
        with urllib.request.urlopen(self.get_url()) as response:
            return response.read()

    def read_csv(self, **kargs):
        """read CSV file from Blob

        Args:
            - path (string): remote csv file path to read
            - **kargs: arguments array passed to pandas.read_csv

        Returns:
            - pandas.DataFrame: DataFrame filled with read datas
        """

        return pd.read_csv(self.get_url(), **kargs)

    def gzip_write(self, name, data):
        return self.write(self.name, gzip.compress(data, compresslevel=9))

    def write(self, name, data):
        return self.storage_account.blob_service.create_blob_from_bytes(
            self.container, self.name, data
        )

    def delete(self):
        return self.storage_account.blob_service.delete_blob(self.container, self.name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from taz.aci import SimpleContainerGroup
from taz.acr import ContainerRegistry
from taz.acr import ContainerImage
import tests.config as cfg
import sys


class AciTests(unittest.TestCase):
    def setUp(self):

        self.container_registry = ContainerRegistry(
            cfg.acr["resource_group"],
            cfg.acr["registry_name"],
            subscription_id=cfg.acr["subscription_id"],
        )

        self.container_image = ContainerImage(
            cfg.acr["image_name"], self.container_registry
        )

        self.container_group = SimpleContainerGroup(
            cfg.aci["container_group_name"],
            cfg.aci["resource_group"],
            cfg.aci["location"],
            self.container_image,
            subscription_id=cfg.aci["subscription_id"],
        )

    def test_exists(self):
        self.assertTrue(self.container_registry is not None)
        self.assertTrue(self.container_image is not None)
        self.assertTrue(self.container_group is not None)

    def test_create(self):
        self.container_group.create()
        self.assertTrue(self.container_group.get_group() is not None)
        self.assertTrue(len(self.container_group.instances) == 1)
        print(self.container_group.auth_method)
        print(self.container_group)

    def test_delete(self):
        self.container_group.delete()


if __name__ == "__main__":
    sys.argv.append("-v")
    unittest.main()



from setuptools import setup, find_packages
import os
from taz._version import __version__

long_description = 'SEE README file'

setup(name='cfa-taz',  # name your package
      packages=find_packages(exclude=['tests/config.py']),
      version=__version__,
      description='Hight level API for azure',
      long_description=long_description,
      package_dir={'taz': 'taz'},
      author='Christophe Fauchard',
      author_email='christophe.fauchard@gmail.com',
      license='MIT',  # MIT, GPL, BSD ??
      install_requires=[
          'pandas>=0.25.3',
          'azure-common==1.1.23',
          'msrestazure==0.6.2',
          'adal==1.2.2',
          'azure-cli-core==2.0.76',
          'azure-mgmt-containerinstance==1.5.0',
          'azure-mgmt-msi==1.0.0',
          'azure-mgmt-containerregistry==2.8.0',
          'azure-storage-common==1.4.2',
          'azure-storage-blob==1.5.0',
          'azure-datalake-store==0.0.48',
          'azure-keyvault-secrets==4.0.0',
          'azure-identity==1.1.0'
      ],
)
